@@ -42,6 +42,8 @@
 #include <hildon/hildon-window.h>
 #endif // GLOM_ENABLE_MAEMO
 
+#include <gtk/gtkuimanager.h>
+
 #ifdef GLOM_ENABLE_MAEMO
 namespace {
 	HildonWindow* turn_gtk_window_into_hildon_window(GtkWindow* cobject)
@@ -84,8 +86,8 @@ App_Glom::App_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml
 #endif // !GLOM_ENABLE_CLIENT_ONLY
   m_menu_tables_ui_merge_id(0),
   m_menu_reports_ui_merge_id(0),
+#ifndef GLOM_ENABLE_CLIENT_ONLY  
   m_ui_save_extra_showextras(false),
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   m_ui_save_extra_newdb_selfhosted(false),
 #endif // !GLOM_ENABLE_CLIENT_ONLY
   m_show_sql_debug(false)
@@ -230,6 +232,22 @@ void App_Glom::init_toolbars()
 */
 }
 
+//gtkmm <2.12.7 has a bug that stops this working with non-ASCII text,
+//so we use the C function instead.
+//We could put this in Bakery instead, but it's hard enough just getting updates 
+//into Ubuntu, so we override this Bakery::App method here.
+//TODO: Remove this when gtkmm 2.12.7 is distributed widely-enough.
+void App_Glom::add_ui_from_string(const Glib::ustring& ui_description)
+{
+  GError* error = 0;
+  gtk_ui_manager_add_ui_from_string(m_refUIManager->gobj(), ui_description.c_str(), ui_description.bytes(), &error);
+  if(error)
+  {
+    std::cerr << "App_Glom::add_ui_from_string(): exception: " << error->message << std::endl;
+    g_clear_error(&error);
+  }
+}
+
 void App_Glom::init_menus_file()
 {
   //Overridden to remove the Save and Save-As menu items,
@@ -558,6 +576,9 @@ Glib::ustring App_Glom::get_file_uri_without_extension(const Glib::ustring& uri)
   }
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+
+
 Glib::ustring App_Glom::ui_file_select_save(const Glib::ustring& old_file_uri) //override
 {
   //Reimplement this whole function, just so we can use our custom FileChooserDialog class:
@@ -654,7 +675,7 @@ Glib::ustring App_Glom::ui_file_select_save(const Glib::ustring& old_file_uri) /
 
     const int response_id = fileChooser_Save->run();
     fileChooser_Save->hide();
-    if(response_id != Gtk::RESPONSE_CANCEL)
+    if((response_id != Gtk::RESPONSE_CANCEL) && (response_id != Gtk::RESPONSE_DELETE_EVENT))
     {
       const Glib::ustring uri_chosen = fileChooser_Save->get_uri();
 
@@ -761,7 +782,6 @@ Glib::ustring App_Glom::ui_file_select_save(const Glib::ustring& old_file_uri) /
   return Glib::ustring();
 }
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
 void App_Glom::stop_self_hosting_of_document_database()
 {
   Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
@@ -771,7 +791,7 @@ void App_Glom::stop_self_hosting_of_document_database()
     if(!connection_pool)
       return;
 
-    connection_pool->stop_self_hosting();
+    connection_pool->stop_self_hosting(this);
   }
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
@@ -904,7 +924,7 @@ bool App_Glom::on_document_load()
 
         const int response = dialog.run();
         dialog.hide();
-        if(response == Gtk::RESPONSE_CANCEL)
+        if((response == Gtk::RESPONSE_CANCEL) || (response == Gtk::RESPONSE_DELETE_EVENT))
           return false;
       }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
@@ -925,7 +945,7 @@ bool App_Glom::on_document_load()
 
           if(!is_example) /* It will be started later, after we have asked for the initial db name/title and created the files.*/
           {
-            const bool test = connection_pool->start_self_hosting(); //Stopped in on_menu_file_close().
+            const bool test = connection_pool->start_self_hosting(this); //Stopped in on_menu_file_close().
             if(!test)
               return false;
           }
@@ -1135,13 +1155,15 @@ bool App_Glom::offer_new_or_existing()
   Gtk::Frame* recent_frame = NULL;
   refXml->get_widget("existing_or_new_recentchooser_frame", recent_frame);
 
-  // Hide the recent chooser when they are not any recently used files
-  if(recent_chooser->get_items().empty()) recent_frame->hide();
+  // Hide the recent chooser when there are no recently used files
+  if(recent_chooser->get_items().empty())
+    recent_frame->hide();
+
   recent_chooser->signal_item_activated().connect(sigc::bind(sigc::mem_fun(*dialog, &Gtk::Dialog::response), 1)); // Open
 
 #ifdef GLOM_ENABLE_CLIENT_ONLY
-  // Don't offer the user to create a new document, because without
-  // developer mode he couldn't do anything useful without it, anyway.
+  // Don't offer the user the choice to create a new document, because without
+  // developer mode he couldn't do anything useful with it anyway:
   Gtk::Button* new_button;
   refXml->get_widget("existing_or_new_button_new", new_button);
   new_button->hide();
@@ -1168,7 +1190,7 @@ bool App_Glom::offer_new_or_existing()
 #endif
 
   const int response_id = dialog->run();
-  Glib::ustring selected_uri = recent_chooser->get_current_uri();
+  const Glib::ustring selected_uri = recent_chooser->get_current_uri();
 
   delete dialog;
   dialog = 0;
@@ -1200,8 +1222,21 @@ bool App_Glom::offer_new_or_existing()
     ui_bring_to_front();
 
     //Ask user to choose file to open:
+    //Create a URI (prefixed by file://) for this path).
+    //Previous versions of gnome-vfs could handle this without file://,
+    //but that stopped working at some point around Ubuntu Hardy. murrayc.
     //g_warning("GLOM_EXAMPLES_DIR=%s", GLOM_EXAMPLES_DIR);
-    Glib::ustring file_uri = ui_file_select_open(GLOM_EXAMPLES_DIR);
+    Glib::ustring examples_uri;
+    try
+    {
+      examples_uri = Glib::filename_to_uri(GLOM_EXAMPLES_DIR);
+    }
+    catch(const Glib::Error& ex)
+    {
+      std::cerr << "Glom: Error converting examples path to a URI: " << ex.what() << std::endl;
+    }
+    
+    Glib::ustring file_uri = ui_file_select_open(examples_uri);
     if(!file_uri.empty())
       open_document(file_uri);
 
@@ -1307,7 +1342,7 @@ bool App_Glom::offer_new_or_existing()
     }
   }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
-  else if(response_id == Gtk::RESPONSE_CANCEL)
+  else if((response_id == Gtk::RESPONSE_CANCEL) || (response_id == Gtk::RESPONSE_DELETE_EVENT))
   {
     return false; //close the window to close the application, because they need to choose a new or existing document.
   }
@@ -1663,7 +1698,8 @@ void App_Glom::fill_menu_tables()
   {
     const Glib::Error& ex = *error.get();
 #endif // GLIBMM_EXCEPTIONS_ENABLED
-    std::cerr << " App_Glom::fill_menu_tables(): building menus failed: " <<  ex.what();
+    std::cerr << " App_Glom::fill_menu_tables(): building menus failed: " <<  ex.what() << std::endl;
+    std::cerr << "   The ui_description was: " <<  ui_description << std::endl;
   }
 }
 
