@@ -21,6 +21,8 @@
 #ifndef BASE_DB_H
 #define BASE_DB_H
 
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
+
 #include "gtkmm.h"
 
 #include <glom/libglom/document/document_glom.h>
@@ -81,7 +83,11 @@ public:
   virtual AppState::userlevels get_userlevel() const;
   virtual void set_userlevel(AppState::userlevels value);
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   static sharedptr<SharedConnection> connect_to_server(Gtk::Window* parent_window = 0);
+#else
+  static sharedptr<SharedConnection> connect_to_server(Gtk::Window* parent_window, std::auto_ptr<ExceptionConnection>& error);
+#endif // GLIBMM_EXCEPTIONS_ENABLED
 
   virtual void set_document(Document_Glom* pDocument); //View override
   virtual void load_from_document(); //View override
@@ -112,10 +118,12 @@ public:
 protected:
   sharedptr<LayoutItem_Field> offer_field_list(const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
   sharedptr<LayoutItem_Field> offer_field_list(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   sharedptr<LayoutItem_Field> offer_field_formatting(const sharedptr<const LayoutItem_Field>& start_field, const Glib::ustring& table_name, Gtk::Window* transient_for = 0);
   sharedptr<LayoutItem_Text> offer_textobject(const sharedptr<LayoutItem_Text>& start_textobject, Gtk::Window* transient_for = 0);
   sharedptr<LayoutItem_Image> offer_imageobject(const sharedptr<LayoutItem_Image>& start_imageobject, Gtk::Window* transient_for = 0);
   sharedptr<LayoutItem_Notebook> offer_notebook(const sharedptr<LayoutItem_Notebook>& start_notebook, Gtk::Window* transient_for = 0);
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   ///@result Whether the user would like to find again.
   static bool show_warning_no_records_found(Gtk::Window& transient_for);
@@ -328,7 +336,7 @@ protected:
 
   static type_vecStrings util_vecStrings_from_Fields(const type_vecFields& fields);
 
-
+  static void handle_error(const Glib::Exception& ex);
   static void handle_error(const std::exception& ex); //TODO_port: This is probably useless now.
   static bool handle_error();
 
