@@ -25,14 +25,17 @@
 
 #include <sstream> //For stringstream
 
+namespace Glom
+{
+
 Box_DB::Box_DB()
-: m_Box_Buttons(false, 6),
+: m_Box_Buttons(false, Utils::DEFAULT_SPACING_SMALL),
   m_Button_Cancel(Gtk::Stock::CANCEL)
 {
   //m_pDocument = 0;
 
-  set_border_width(6);
-  set_spacing(6);
+  set_border_width(Utils::DEFAULT_SPACING_SMALL);
+  set_spacing(Utils::DEFAULT_SPACING_SMALL);
 
   //Connect signals:
   m_Button_Cancel.signal_clicked().connect(sigc::mem_fun(*this, &Box_DB::on_Button_Cancel));
@@ -40,13 +43,13 @@ Box_DB::Box_DB()
 
 Box_DB::Box_DB(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& /* refGlade */)
 : Gtk::VBox(cobject),
-  m_Box_Buttons(false, 6),
+  m_Box_Buttons(false, Utils::DEFAULT_SPACING_SMALL),
   m_Button_Cancel(Gtk::Stock::CANCEL)
 {
   //m_pDocument = 0;
 
-  set_border_width(6);
-  set_spacing(6);
+  set_border_width(Utils::DEFAULT_SPACING_SMALL);
+  set_spacing(Utils::DEFAULT_SPACING_SMALL);
 
   //Connect signals:
   m_Button_Cancel.signal_clicked().connect(sigc::mem_fun(*this, &Box_DB::on_Button_Cancel));
@@ -54,7 +57,7 @@ Box_DB::Box_DB(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& /
 
 Box_DB::Box_DB(BaseObjectType* cobject)
 : Gtk::VBox(cobject),
-  m_Box_Buttons(false, 6),
+  m_Box_Buttons(false, Utils::DEFAULT_SPACING_SMALL),
   m_Button_Cancel(Gtk::Stock::CANCEL)
 {
 }
@@ -119,3 +122,5 @@ Gtk::Widget* Box_DB::get_default_button()
 }
 
 
+} //namespace Glom
+
