@@ -35,7 +35,8 @@
 
 #include <sys/types.h>
 #include <sys/socket.h> 
-#include <sys/socket.h>
+#include <errno.h>
+
 #include <netinet/in.h> //For sockaddr_in
 
 #include <signal.h> //To catch segfaults
@@ -655,14 +656,19 @@ static void on_linux_signal(int signum)
 
   if(signum == SIGSEGV)
   {
-    connection_pool->stop_self_hosting();
+    //TODO: Make this dialog transient for the parent window, 
+    //though this is obviously an unusual case.
+    connection_pool->stop_self_hosting(0 /* parent_window */);
 	
-    //TODO: How can we let GNOME's crash handler still handle this?
-    exit(1);
+    //Let GNOME/Ubuntu's crash handler still handle this?
+    if(previous_sig_handler)
+      (*previous_sig_handler)(signum);
+    else
+      exit(1);
   }
 }
 
-bool ConnectionPool::start_self_hosting()
+bool ConnectionPool::start_self_hosting(Gtk::Window* parent_window)
 {
   if(m_self_hosting_active)
     return true; //Just do it once.
@@ -724,7 +730,7 @@ bool ConnectionPool::start_self_hosting()
   const std::string second_command_success_text = "is running"; //TODO: This is not a stable API. Also, watch out for localisation.
 
   //The first command does not return, but the second command can check whether it succeeded:
-  const bool result = Glom::Spawn::execute_command_line_and_wait_until_second_command_returns_success(command_postgres_start, command_check_postgres_has_started, _("Starting Database Server"), 0 /* window*/, second_command_success_text);
+  const bool result = Glom::Spawn::execute_command_line_and_wait_until_second_command_returns_success(command_postgres_start, command_check_postgres_has_started, _("Starting Database Server"), parent_window, second_command_success_text);
   if(!result)
   {
     std::cerr << "Error while attempting to self-host a database." << std::endl;
@@ -743,7 +749,7 @@ bool ConnectionPool::start_self_hosting()
   return true;
 }
 
-void ConnectionPool::stop_self_hosting()
+void ConnectionPool::stop_self_hosting(Gtk::Window* parent_window)
 {
   if(!m_self_hosting_active)
     return; //Don't try to stop it if we have not started it.
@@ -765,15 +771,17 @@ void ConnectionPool::stop_self_hosting()
   // -c config_file= specifies the configuration file
   // -k specifies a directory to use for the socket. This must be writable by us.
   // POSTGRES_POSTMASTER_PATH is defined in config.h, based on the configure.
-  const std::string command_postgres_stop = POSTGRES_UTILS_PATH "/pg_ctl -D \"" + dbdir_data + "\" stop";
-  const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, _("Stopping Database Server"));
+  // We use "-m fast" instead of the default "-m smart" because that waits for clients to disconnect (and sometimes never succeeds).
+  // TODO: Warn about connected clients on other computers? Warn those other users?
+  const std::string command_postgres_stop = POSTGRES_UTILS_PATH "/pg_ctl -D \"" + dbdir_data + "\" stop -m fast";
+  const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, _("Stopping Database Server"), parent_window);
   if(!result)
   {
     std::cerr << "Error while attempting to stop self-hosting of the database. Trying again."  << std::endl;
 
     //I've seen it fail when running under valgrind, and there are reports of failures in bug #420962.
     //Maybe it will help to try again:
-    const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, _("Stopping Database Server (retrying)"));
+    const bool result = Glom::Spawn::execute_command_line_and_wait(command_postgres_stop, _("Stopping Database Server (retrying)"), parent_window);
     if(!result)
     {
       std::cerr << "Error while attempting (for a second time) to stop self-hosting of the database."  << std::endl;
@@ -872,7 +880,7 @@ bool ConnectionPool::create_self_hosting(Gtk::Window* parent_window)
   const std::string command_initdb = POSTGRES_UTILS_PATH "/initdb -D \"" + dbdir_data + "\"" +
                                         " -U " + username + " --pwfile=\"" + temp_pwfile + "\""; 
   //Note that --pwfile takes the password from the first line of a file. It's an alternative to supplying it when prompted on stdin.
-  const bool result = Glom::Spawn::execute_command_line_and_wait(command_initdb, _("Creating Database Data"));
+  const bool result = Glom::Spawn::execute_command_line_and_wait(command_initdb, _("Creating Database Data"), parent_window);
   if(!result)
   {
     std::cerr << "Error while attempting to create self-hosting database." << std::endl;
@@ -971,31 +979,53 @@ int ConnectionPool::discover_first_free_port(int start_port, int end_port)
     sa.sin_port = htons(port_to_try);
 
     const int result = bind(fd, (sockaddr*)&sa, sizeof(sa));
-    if((result == 0) || ((result < 0)
-#ifdef EADDRINUSE //Some BSDs don't have this.
-    && (errno != EADDRINUSE)
-#endif 
-#ifdef EPORTINUSE //Linux doesn't have this.
-    && (errno != EPORTINUSE)
-#endif
-    ))
+    bool available = false;
+    if(result == 0)
+       available = true;
+    else if (result < 0)
     {
-      close(fd);
+      #ifdef G_OS_WIN32
+      available = (WSAGetLastError() != WSAEADDRINUSE);
+      #endif // G_OS_WIN32
 
-      std::cout << "debug: ConnectionPool::discover_first_free_port(): Found: returning " << port_to_try << std::endl;
-      return port_to_try;
+      //Some BSDs don't have this.
+      //But watch out - if you don't include errno.h then this won't be 
+      //defined on Linux either, but you really do need to check for it.
+      #ifdef EADDRINUSE
+      available = (errno != EADDRINUSE);
+      #endif
+
+      #ifdef EPORTINUSE //Linux doesn't have this.
+      available = (errno != EPORTINUSE);
+      #endif
+
+      if(available)
+      {
+        #ifdef G_OS_WIN32
+        closesocket(fd);
+        #else
+        close(fd);
+        #endif //G_OS_WIN32
+
+        //std::cout << "debug: ConnectionPool::discover_first_free_port(): Found: returning " << port_to_try << std::endl;
+        return port_to_try;
+      }
     }
     else
     {
-      std::cout << "debug: ConnectionPool::discover_first_free_port(): port in use: " << port_to_try << std::endl;
+      //std::cout << "debug: ConnectionPool::discover_first_free_port(): port in use: " << port_to_try << std::endl;
     }
 
     ++port_to_try;
   }
 
+#ifdef G_OS_WIN32
+  closesocket(fd);
+#else
   close(fd);
+#endif
 
-  std::cout << "debug: ConnectionPool::discover_first_free_port(): No port was available." << std::endl;
+  std::cerr << "debug: ConnectionPool::discover_first_free_port(): No port was available." << std::endl;
   return 0;
 }
 #endif // !GLOM_ENABLE_CLIENT_ONLY
