@@ -20,7 +20,7 @@
 
 #include "dialog_layout_details.h"
 #include "dialog_choose_relationship.h"
-#include "../mode_design/dialog_buttonscript.h"
+#include "../layout_item_dialogs/dialog_buttonscript.h"
 #include "../layout_item_dialogs/dialog_notebook.h"
 #include "../frame_glom.h" //For show_ok_dialog()
 //#include <libgnome/gnome-i18n.h>
@@ -28,10 +28,16 @@
 #include <glibmm/i18n.h>
 #include <sstream> //For stringstream
 
+namespace Glom
+{
+
 Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
 : Dialog_Layout(cobject, refGlade),
   m_treeview_fields(0),
   m_treeview_column_title(0),
+  m_box_table_widgets(0),
+  m_box_related_table_widgets(0),
+  m_box_related_navigation(0),
   m_button_field_up(0),
   m_button_field_down(0),
   m_button_field_add(0),
@@ -46,6 +52,15 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_edit(0),
   m_label_table_name(0)
 {
+  // Get the alternate sets of widgets, only one of which should be shown:
+  // Derived classes will hide one and show the other:
+  refGlade->get_widget("hbox_table_widgets", m_box_table_widgets);
+  m_box_table_widgets->show();
+  refGlade->get_widget("hbox_related_table_widgets", m_box_related_table_widgets);
+  m_box_related_table_widgets->hide();
+  refGlade->get_widget("frame_related_table_navigation", m_box_related_navigation); 
+  m_box_related_navigation->hide();
+
   refGlade->get_widget("label_table_name", m_label_table_name);
 
   refGlade->get_widget("treeview_fields", m_treeview_fields);
@@ -143,7 +158,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   refGlade->get_widget("button_edit", m_button_edit);
   m_button_edit->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_edit) );
 
-  show_all_children();
+  //show_all_children();
 }
 
 Dialog_Layout_Details::~Dialog_Layout_Details()
@@ -311,7 +326,7 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
     }
 
     //Show the field layout
-    typedef std::list< Glib::ustring > type_listStrings;
+    //typedef std::list< Glib::ustring > type_listStrings;
 
     m_model_items->clear();
 
@@ -320,6 +335,8 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
     {
       sharedptr<const LayoutGroup> group = iter->second;
+      //std::cout << "debug: Dialog_Layout_Details::set_document(): adding group name=" << group->get_name() << std::endl;
+        
       sharedptr<const LayoutGroup> portal = sharedptr<const LayoutItem_Portal>::cast_dynamic(group);
       if(group && !portal)
         add_group(Gtk::TreeModel::iterator() /* null == top-level */, group);
@@ -524,7 +541,7 @@ sharedptr<LayoutItem_Button> Dialog_Layout_Details::offer_button_script_edit(con
     {
       dialog->set_script(button, m_table_name);
       dialog->set_transient_for(*this);
-      const int response = dialog->run();
+      int response = Glom::Utils::dialog_run_with_help(dialog, "window_button_script");
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
       {
@@ -581,7 +598,12 @@ sharedptr<LayoutItem_Text> Dialog_Layout_Details::offer_textobject_edit(const sh
 
 sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list()
 {
-  sharedptr<Relationship> result;
+  return offer_relationship_list(sharedptr<Relationship>());
+}
+
+sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list(const sharedptr<const Relationship>& item)
+{
+  sharedptr<Relationship> result = glom_sharedptr_clone(item);
 
   try
   {
@@ -593,6 +615,7 @@ sharedptr<Relationship> Dialog_Layout_Details::offer_relationship_list()
     if(dialog)
     {
       dialog->set_document(get_document(), m_table_name);
+      dialog->select_item(item);
       dialog->set_transient_for(*this);
       const int response = dialog->run();
       dialog->hide();
@@ -864,6 +887,7 @@ void Dialog_Layout_Details::on_button_field_formatting()
         if(chosenitem)
         {
           *field = *chosenitem; //TODO_Performance.
+          m_modified = true;
           //m_model_parts->row_changed(Gtk::TreePath(iter), iter); //TODO: Add row_changed(iter) to gtkmm?
         }
       }
@@ -890,13 +914,14 @@ void Dialog_Layout_Details::on_button_edit()
       sharedptr<LayoutItem_Portal> layout_portal = sharedptr<LayoutItem_Portal>::cast_dynamic(layout_item);
       if(layout_portal)
       {
-        sharedptr<Relationship> relationship(new Relationship());
-        relationship->set_name( layout_portal->get_relationship_name() ); //TODO: Start with this one selected.
-        relationship = offer_relationship_list();
+        sharedptr<Relationship> relationship = offer_relationship_list(layout_portal->get_relationship());
         if(relationship)
         {
           layout_portal->set_relationship(relationship);
-          row[m_model_items->m_columns.m_col_layout_item] = layout_portal;
+ 
+          //This is unnecessary and seems to cause a crash.
+          //row[m_model_items->m_columns.m_col_layout_item] = layout_portal;
+
           m_modified = true;
         }
       }
@@ -931,7 +956,8 @@ void Dialog_Layout_Details::on_button_edit()
               sharedptr<LayoutItem_Field> chosenitem = offer_field_list(layout_item_field, m_table_name, this);
               if(chosenitem)
               {
-                row[m_model_items->m_columns.m_col_layout_item] = chosenitem;
+                *layout_item_field = *chosenitem;
+                //row[m_model_items->m_columns.m_col_layout_item] = chosenitem;
                 m_modified = true;
               }
             }
@@ -983,6 +1009,12 @@ void Dialog_Layout_Details::on_button_edit()
           }
         }
       }
+
+      if(m_modified)
+      {
+        Gtk::TreeModel::Path path = m_model_items->get_path(iter);
+        m_model_items->row_changed(path, iter);
+      }
     }
   }
 }
@@ -1020,10 +1052,12 @@ void Dialog_Layout_Details::save_to_document()
       if(layout_group && !layout_portal) //There may be top-level groups, but no top-level fields, because the fields must be in a group (so that they are in columns)
       {
         sharedptr<LayoutGroup> group = sharedptr<LayoutGroup>::create();
-        group->m_sequence = group_sequence;
         fill_group(iterFields, group);
 
+        group->m_sequence = group_sequence;
         mapGroups[group_sequence] = group;
+        //std::cout << "debug: Dialog_Layout_Details::save_to_document(): adding group sequence=" << group_sequence << ", group->m_sequence=" << group->m_sequence << ". group name=" << group->get_name() << std::endl;
+        
         ++group_sequence;
       }
     }
@@ -1274,6 +1308,8 @@ void Dialog_Layout_Details::on_treeview_cell_edited_columns_count(const Glib::us
   }
 }
 
+} //namespace Glom
+
 
 
 
