@@ -25,6 +25,9 @@
 #include <glom/libglom/data_structure/layout/layoutitem_portal.h>
 #include <glom/libglom/data_structure/layout/layoutitem_button.h>
 
+namespace Glom
+{
+
 class TreeStore_Layout : public Gtk::TreeStore
 {
 protected:
@@ -58,5 +61,7 @@ protected:
 
 };
 
+} //namespace Glom
+
 #endif //GLOM_MODE_DATA_TREESTORE_LAYOUT_H
 
