@@ -23,17 +23,22 @@
 
 #include "adddel.h"
 
+namespace Glom
+{
+
 class AddDel_WithButtons : public AddDel
 {
 public: 
   AddDel_WithButtons();
+  AddDel_WithButtons(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~AddDel_WithButtons();
 
   virtual void set_allow_add(bool val = true); //override
   virtual void set_allow_delete(bool val = true); //override
   virtual void set_allow_user_actions(bool bVal = true); //override
-  
+
 protected:
+  void init();
   virtual void setup_buttons();
 
   virtual void on_button_add();
@@ -47,4 +52,6 @@ protected:
   Gtk::Button m_Button_Edit;
 };
 
+} //namespace Glom
+
 #endif //ADDDEL_WITHBUTTONS_H
