@@ -33,6 +33,7 @@
 //#include "../cellrendererlist.h"
 #include <iostream> //For debug output.
 #include <gtk/gtktreeview.h>
+#include <gtk/gtkstock.h>
 
 namespace Glom
 {
@@ -82,7 +83,7 @@ DbAddDel::DbAddDel()
   set_prevent_user_signals();
   set_ignore_treeview_signals(true);
 
-  set_spacing(6);
+  set_spacing(Utils::DEFAULT_SPACING_SMALL);
 
   //Start with a useful default TreeModel:
   //set_columns_count(1);
@@ -94,6 +95,7 @@ DbAddDel::DbAddDel()
   m_ScrolledWindow.set_shadow_type(Gtk::SHADOW_IN);
 
   m_TreeView.set_fixed_height_mode(); //This allows some optimizations.
+  m_TreeView.set_headers_visible(true); //This is not the default in Maemo's GTK+. Bad Maemo.
   m_TreeView.show();
 
   pack_start(m_ScrolledWindow);
@@ -120,15 +122,20 @@ DbAddDel::DbAddDel()
   remove_all_columns(); //set up the default columns.
 
   show_all_children();
+
+  // Adjust sizing when style changed
+  signal_style_changed().connect(sigc::mem_fun(*this, &DbAddDel::on_self_style_changed));
 }
 
 DbAddDel::~DbAddDel()
 {
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   App_Glom* pApp = get_application();
   if(pApp)
   {
     pApp->remove_developer_action(m_refContextLayout);
   } 
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 }
 
 void
@@ -170,11 +177,13 @@ DbAddDel::on_MenuPopup_activate_Edit()
   }
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 void DbAddDel::on_MenuPopup_activate_layout()
 {
   finish_editing();
   signal_user_requested_layout().emit();
 }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 void DbAddDel::on_MenuPopup_activate_Add()
 {
@@ -231,6 +240,9 @@ void DbAddDel::setup_menu()
     sigc::mem_fun(*this, &DbAddDel::on_MenuPopup_activate_Add) );
   m_refContextAdd->set_sensitive(m_allow_add);
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  // Don't add ContextLayout in client only mode because it would never
+  // be sensitive anyway
   m_refContextLayout =  Gtk::Action::create("ContextLayout", _("Layout"));
   m_refActionGroup->add(m_refContextLayout,
     sigc::mem_fun(*this, &DbAddDel::on_MenuPopup_activate_layout) );
@@ -242,6 +254,7 @@ void DbAddDel::setup_menu()
     pApp->add_developer_action(m_refContextLayout); //So that it can be disabled when not in developer mode.
     pApp->update_userlevel_ui(); //Update our action's sensitivity. 
   }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   m_refUIManager = Gtk::UIManager::create();
 
@@ -249,24 +262,37 @@ void DbAddDel::setup_menu()
 
   //TODO: add_accel_group(m_refUIManager->get_accel_group());
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
   try
   {
+#endif
     Glib::ustring ui_info = 
         "<ui>"
         "  <popup name='ContextMenu'>"
         "    <menuitem action='ContextEdit'/>"
         "    <menuitem action='ContextAdd'/>"
         "    <menuitem action='ContextDelete'/>"
-        "    <menuitem action='ContextLayout'/>"        
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+        "    <menuitem action='ContextLayout'/>"
+#endif
         "  </popup>"
         "</ui>";
 
+#ifdef GLIBMM_EXCEPTIONS_ENABLED
     m_refUIManager->add_ui_from_string(ui_info);
   }
   catch(const Glib::Error& ex)
   {
     std::cerr << "building menus failed: " <<  ex.what();
   }
+#else
+  std::auto_ptr<Glib::Error> error;
+  m_refUIManager->add_ui_from_string(ui_info, error);
+  if(error.get() != NULL)
+  {
+    std::cerr << "building menus failed: " << error->what();
+  }
+#endif //GLIBMM_EXCEPTIONS_ENABLED
 
   //Get the menu:
   m_pMenuPopup = dynamic_cast<Gtk::Menu*>( m_refUIManager->get_widget("/ContextMenu") ); 
@@ -285,12 +311,15 @@ void DbAddDel::setup_menu()
     m_refContextDelete->set_sensitive(false);
   }
  
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   if(pApp)
     m_refContextLayout->set_sensitive(pApp->get_userlevel() == AppState::USERLEVEL_DEVELOPER);
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 }
 
 bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
 {
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   //Enable/Disable items.
   //We did this earlier, but get_application is more likely to work now:
   App_Glom* pApp = get_application();
@@ -320,6 +349,7 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
       return false; //Not handled.
     }
   }
+#endif
 
   return  false; //Not handled. TODO: Call base class?
 }
@@ -327,7 +357,10 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
 Gtk::TreeModel::iterator DbAddDel::get_item_placeholder()
 {
   //Get the existing placeholder row, or add one if necessary:
-  return m_refListStore->get_placeholder_row();
+  if(m_refListStore)
+    return m_refListStore->get_placeholder_row();
+  else
+   return Gtk::TreeModel::iterator();
 }
 
 Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item)
@@ -340,7 +373,7 @@ Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, cons
 
     if(treerow)
     {
-      type_list_indexes list_indexes = get_column_index(layout_item);
+      type_list_indexes list_indexes = get_data_model_column_index(layout_item);
       if(!list_indexes.empty())
       {
         type_list_indexes::const_iterator iter = list_indexes.begin(); //Just get the first displayed instance of this field->
@@ -378,7 +411,10 @@ Gtk::TreeModel::iterator DbAddDel::get_item_selected()
      return refTreeSelection->get_selected();
   }
 
-  return m_refListStore->children().end();
+  if(m_refListStore)
+    return m_refListStore->children().end();
+  else
+    return Gtk::TreeModel::iterator();
 }
 
 
@@ -530,12 +566,24 @@ int DbAddDel::get_fixed_cell_height()
   }
 }
 
-void DbAddDel::on_cell_layout_button_clicked(int /* model_column_index */)
+void DbAddDel::on_cell_layout_button_clicked(const Gtk::TreeModel::Path& path, int model_column_index)
 {
+  if(!m_refListStore)
+    return;
 
+  Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
+  if(iter)
+  {
+    sharedptr<const LayoutItem> layout_item = m_ColumnTypes[model_column_index].m_item;
+    sharedptr<const LayoutItem_Button> item_button = sharedptr<const LayoutItem_Button>::cast_dynamic(layout_item);
+    if(item_button)
+    {
+      m_signal_script_button_clicked.emit(item_button, iter);
+    }
+  }
 }
 
-Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const sharedptr<LayoutItem>& layout_item, int model_column_index)
+Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const sharedptr<LayoutItem>& layout_item, int model_column_index, int data_model_column_index)
 {
   Gtk::CellRenderer* pCellRenderer = 0;
 
@@ -587,9 +635,9 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
 
        Glib::RefPtr<Gdk::Pixbuf> pixbuf = Conversions::get_pixbuf_for_gda_value(item_image->get_image());
        if(pixbuf)
-         pixbuf_renderer->property_pixbuf() = pixbuf;
+         pixbuf_renderer->set_property("pixbuf", pixbuf);
        else
-         pixbuf_renderer->property_stock_id() = Gtk::Stock::MISSING_IMAGE.id; //TODO: Specify stock size?
+         pixbuf_renderer->set_property("stock-id", Gtk::Stock::MISSING_IMAGE);
 
        pCellRenderer = pixbuf_renderer;
      }
@@ -599,7 +647,7 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
        if(item_text)
        {
          Gtk::CellRendererText* pCellText = Gtk::manage( new Gtk::CellRendererText() );
-         pCellText->property_text() = item_text->get_text();
+	 pCellText->set_property("text", item_text->get_text());
 
          pCellRenderer = pCellText;
        }
@@ -609,11 +657,11 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
          if(item_button)
          {
            GlomCellRenderer_ButtonText* pCellButton = Gtk::manage( new GlomCellRenderer_ButtonText() );
-           pCellButton->property_text() = item_button->get_title_or_name();
+	   pCellButton->set_property("text", item_button->get_title_or_name());
            //pCellButton->set_fixed_width(50); //Otherwise it doesn't show up. TODO: Discover the width of the contents.
 
-           //TODO: pCellButton->signal_clicked().connect(
-           //  sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_cell_layout_button_clicked), model_column_index) );
+           pCellButton->signal_clicked().connect(
+             sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_cell_layout_button_clicked), model_column_index) );
 
            pCellRenderer = pCellButton;
          }
@@ -629,7 +677,7 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
     //Use an ellipze to indicate excessive text, 
     //so that similar values do not look equal, 
     //and to avoid multi-line comments. TODO: Is there a better way to restrict the height? This doesn't actually truncate multilines anyway.
-    pCellRendererText->property_ellipsize() = Pango::ELLIPSIZE_END;
+    g_object_set(pCellRendererText->gobj(), "ellipsize", PANGO_ELLIPSIZE_END, (gpointer)NULL);
 
     //Restrict the height, to prevent multiline text cells,
     //and to allow TreeView performance optimisation:
@@ -639,15 +687,15 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
     if(item_field) //Only fields can be edited:
     {
       //Make it editable:
-      pCellRendererText->property_editable() = true;
+      pCellRendererText->set_property("editable", true);
 
       //Align numbers to the right: //TODO: Avoid this for ID keys.
       if(item_field->get_glom_type() == Field::TYPE_NUMERIC )
-        pCellRenderer->property_xalign() = 1.0f; //Align right.
+        pCellRendererText->set_property("xalign", 1.0);
 
       //Connect to its signal:
       pCellRendererText->signal_edited().connect(
-        sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index) );
+        sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited), model_column_index, data_model_column_index) );
 
     }
 
@@ -715,11 +763,14 @@ Gtk::CellRenderer* DbAddDel::construct_specified_columns_cellrenderer(const shar
     Gtk::CellRendererToggle* pCellRendererToggle = dynamic_cast<Gtk::CellRendererToggle*>(pCellRenderer);
     if(pCellRendererToggle)
     {
-      pCellRendererToggle->property_activatable() = true;
+      pCellRendererToggle->set_property("activatable", true);
 
-      //Connect to its signal:
-      pCellRendererToggle->signal_toggled().connect(
-      sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited_bool), model_column_index ) );
+      if(item_field) //Only fields can be edited:
+      {
+        //Connect to its signal:
+        pCellRendererToggle->signal_toggled().connect(
+        sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_cell_edited_bool), model_column_index, data_model_column_index ) );
+      }
     }
     else
     {
@@ -803,32 +854,31 @@ void DbAddDel::construct_specified_columns()
     ++column_index_key;
   }
 
-  if(!key_found)
+  if(key_found)
   {
-    g_warning("DbAddDel::construct_specified_columns(): no primary key field found.");
+    //Create the model from the ColumnRecord:
+    m_refListStore = type_model_store::create(record, m_found_set, fields, column_index_key, m_allow_view);
+  }
+  else
+  {
+    g_warning("%s: no primary key field found.", __FUNCTION__);
     //for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
     //{
     //  g_warning("  field: %s", (iter->get_name().c_str());
     //}
 
-    g_assert(key_found); //crash here.
+    m_refListStore = Glib::RefPtr<type_model_store>();
   }
 
-  //Create the model from the ColumnRecord:
-  m_refListStore = type_model_store::create(record, m_found_set, fields, column_index_key, m_allow_view);
-
   m_TreeView.set_model(m_refListStore);
 
 
-
-
   //Remove all View columns:
   m_TreeView.remove_all_columns();
 
   //Add new View Colums:
   int model_column_index = 0; //Not including the hidden internal columns.
   int view_column_index = 0;
-
   {
     GlomCellRenderer_ButtonImage* pCellButton = Gtk::manage(new GlomCellRenderer_ButtonImage());
     pCellButton->signal_clicked().connect(sigc::mem_fun(*this, &DbAddDel::on_cell_button_clicked));
@@ -837,9 +887,20 @@ void DbAddDel::construct_specified_columns()
     m_treeviewcolumn_button = Gtk::manage(new Gtk::TreeViewColumn());
     m_treeviewcolumn_button->pack_start(*pCellButton);
 
+
+    int x_offset, y_offset, width, height;
+    pCellButton->get_size(m_TreeView, x_offset, y_offset, width, height);
+
     m_treeviewcolumn_button->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED); //Need by fixed-height mode.
-    m_treeviewcolumn_button->set_fixed_width(25); //Otherwise it doesn't show up. TODO: Discover the width of the contents.
-    m_treeviewcolumn_button->property_visible() = true; //get_allow_view_details();
+
+    // TODO: I am not sure whether this is always correct. Perhaps, we also
+    // have to take into account the xpad property of the cell renderer and
+    // the spacing property of the treeviewcolumn.
+    int horizontal_separator;
+    m_TreeView.get_style_property("horizontal-separator", horizontal_separator);
+    m_treeviewcolumn_button->set_fixed_width(width + horizontal_separator*2);
+
+    m_treeviewcolumn_button->set_property("visible", true);
 
     m_TreeView.append_column(*m_treeviewcolumn_button);
 
@@ -847,22 +908,35 @@ void DbAddDel::construct_specified_columns()
   }
 
   bool no_columns_used = true;
+  int data_model_column_index = 0;
+  
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
     const DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
     if(column_info.m_visible)
     {
-      
       no_columns_used = false;
 
       const Glib::ustring column_name = column_info.m_item->get_title_or_name();
       const Glib::ustring column_id = column_info.m_item->get_name();
 
+      // Whenever we are dealing with real database fields, 
+      // we need to know the index of the field in the query:
+      int item_data_model_column_index = -1;
+      sharedptr<const LayoutItem> layout_item = m_ColumnTypes[model_column_index].m_item;
+      sharedptr<const LayoutItem_Field> item_field = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
+      if(item_field)
+      {
+        item_data_model_column_index = data_model_column_index;
+        ++data_model_column_index;
+      }
+
       //Add the ViewColumn
-      Gtk::CellRenderer* pCellRenderer = construct_specified_columns_cellrenderer(column_info.m_item, model_column_index);
+      Gtk::CellRenderer* pCellRenderer = construct_specified_columns_cellrenderer(column_info.m_item, model_column_index, item_data_model_column_index);
       if(pCellRenderer)
       {
-        treeview_append_column(column_name, *pCellRenderer, model_column_index);
+        //Get the index of the field in the query, if it is a field:
+        treeview_append_column(column_name, *pCellRenderer, model_column_index, item_data_model_column_index);
 
         if(column_info.m_editable)
         {
@@ -947,7 +1021,7 @@ void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const sharedptr<c
     Gtk::TreeModel::Row treerow = *iter;
     if(treerow)
     {
-      type_list_indexes list_indexes = get_column_index(layout_item);
+      type_list_indexes list_indexes = get_data_model_column_index(layout_item);
       for(type_list_indexes::const_iterator iter = list_indexes.begin(); iter != list_indexes.end(); ++iter)
       {
         guint treemodel_col = *iter + get_count_hidden_system_columns();
@@ -992,18 +1066,15 @@ guint DbAddDel::add_column(const sharedptr<LayoutItem>& layout_item)
     return 0; //TODO: Do something more sensible.
 
   InnerIgnore innerIgnore(this); //Stop on_treeview_columns_changed() from doing anything when it is called just because we add a new column.
-
-  sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
-  if(!field)
-    return 0; //TODO: Do something more sensible.
-
+  
   DbAddDelColumnInfo column_info;
-  column_info.m_item = field;
+  column_info.m_item = layout_item;
   //column_info.m_editable = editable;
   //column_info.m_visible = visible;
 
   //Make it non-editable if it is auto-generated:
 
+  sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(layout_item);
   if(field)
   {
     sharedptr<const Field> field_full = field->get_full_field_details();
@@ -1041,6 +1112,33 @@ void DbAddDel::set_columns_ready()
   construct_specified_columns();
 }
 
+DbAddDel::type_list_indexes DbAddDel::get_data_model_column_index(const sharedptr<const LayoutItem_Field>& layout_item_field) const
+{
+  //TODO_Performance: Replace all this looping by a cache/map:
+
+  type_list_indexes list_indexes;
+
+  if(!layout_item_field)
+    return list_indexes;
+
+  guint data_model_column_index = 0;
+  for(type_ColumnTypes::const_iterator iter = m_ColumnTypes.begin(); iter != m_ColumnTypes.end(); ++iter)
+  {
+    sharedptr<const LayoutItem_Field> field = sharedptr<const LayoutItem_Field>::cast_dynamic(iter->m_item); //TODO_Performance: This would be unnecessary if !layout_item_field
+    if(field)
+    {
+      if(field && field->is_same_field(layout_item_field))
+      {
+        list_indexes.push_back(data_model_column_index);
+      }
+
+      ++data_model_column_index;
+    }
+  }
+
+  return list_indexes;
+}
+
 DbAddDel::type_list_indexes DbAddDel::get_column_index(const sharedptr<const LayoutItem>& layout_item) const
 {
   //TODO_Performance: Replace all this looping by a cache/map:
@@ -1187,7 +1285,7 @@ void DbAddDel::reactivate()
 
 void DbAddDel::remove_item(const Gtk::TreeModel::iterator& iter)
 {
-  if(iter)
+  if(iter && m_refListStore)
     m_refListStore->erase(iter);
 }
 
@@ -1240,13 +1338,16 @@ Gnome::Gda::Value DbAddDel::treeview_get_key(const Gtk::TreeModel::iterator& row
   return value;
 }
 
-void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index)
+void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index, int data_model_column_index)
 {
   //Note:: model_column_index is actually the AddDel column index, not the TreeModel column index.
 
   if(path_string.empty())
     return;
 
+  if(!m_refListStore)
+    return;
+
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
@@ -1255,12 +1356,12 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
   {
     Gtk::TreeModel::Row row = *iter;
 
-    int tree_model_column_index = model_column_index + get_count_hidden_system_columns();
+    int tree_model_column_index = data_model_column_index + get_count_hidden_system_columns();
 
     Gnome::Gda::Value value_old;
     row.get_value(tree_model_column_index, value_old);
 
-    const bool bValueOld = (value_old.get_value_type() == Gnome::Gda::VALUE_TYPE_BOOLEAN) && value_old.get_bool();
+    const bool bValueOld = (value_old.get_value_type() == G_TYPE_BOOLEAN) && value_old.get_boolean();
     bool bValueNew = !bValueOld;
     Gnome::Gda::Value value_new;
     value_new.set(bValueNew);
@@ -1274,7 +1375,7 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
     bool bIsAdd = false;
     bool bIsChange = false;
 
-    int iCount = m_refListStore->children().size();
+    const int iCount = m_refListStore->children().size();
     if(iCount)
     {
       if(get_allow_user_actions()) //If add is possible:
@@ -1321,12 +1422,15 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
   }
 }
 
-void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index)
+void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index, int data_model_column_index)
 {
   //Note:: model_column_index is actually the AddDel column index, not the TreeModel column index.
   if(path_string.empty())
     return;
 
+  if(!m_refListStore)
+    return;
+
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
@@ -1335,7 +1439,7 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
   {
     Gtk::TreeModel::Row row = *iter;
 
-    const int treemodel_column_index = model_column_index + get_count_hidden_system_columns();
+    const int treemodel_column_index = data_model_column_index + get_count_hidden_system_columns();
 
     Gnome::Gda::Value valOld;
     row.get_value(treemodel_column_index, valOld);
@@ -1471,10 +1575,12 @@ DbAddDel::type_signal_user_changed DbAddDel::signal_user_changed()
   return m_signal_user_changed;
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
 DbAddDel::type_signal_user_requested_layout DbAddDel::signal_user_requested_layout()
 {
   return m_signal_user_requested_layout;
 }
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
 DbAddDel::type_signal_user_requested_delete DbAddDel::signal_user_requested_delete()
 {
@@ -1501,6 +1607,11 @@ DbAddDel::type_signal_user_reordered_columns DbAddDel::signal_user_reordered_col
   return m_signal_user_reordered_columns;
 }
 
+DbAddDel::type_signal_script_button_clicked DbAddDel::signal_script_button_clicked()
+{
+  return m_signal_script_button_clicked;
+}
+
 void DbAddDel::on_treeview_button_press_event(GdkEventButton* event)
 {
   if(event->type == GDK_BUTTON_PRESS) //Whatever would cause cellrenderer activation.
@@ -1513,11 +1624,16 @@ void DbAddDel::on_treeview_button_press_event(GdkEventButton* event)
     Gtk::TreeView::Column* pColumn = 0;
     int cell_x = 0;
     int cell_y = 0;  
-    bool row_exists = m_TreeView.get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
+
+    // Make sure to use the non-deprecated const version:
+    bool row_exists = static_cast<const Gtk::TreeView&>(m_TreeView).get_path_at_pos((int)event->x, (int)event->y, path, pColumn, cell_x, cell_y);
 
     //Get the row:
     if(row_exists)
     {
+      if(!m_refListStore)
+        return;
+
       Gtk::TreeModel::iterator iterRow = m_refListStore->get_iter(path);
       if(iterRow)
       {
@@ -1560,24 +1676,28 @@ bool DbAddDel::on_treeview_column_drop(Gtk::TreeView* /* treeview */, Gtk::TreeV
   return true;
 }
 
-guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRenderer& cellrenderer, int model_column_index)
+guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRenderer& cellrenderer, int model_column_index, int data_model_column_index)
 {
   DbTreeViewColumnGlom* pViewColumn = Gtk::manage( new DbTreeViewColumnGlom(Utils::string_escape_underscores(title), cellrenderer) );
   pViewColumn->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED); //Need by fixed-height mode.
 
   guint cols_count = m_TreeView.append_column(*pViewColumn);
 
+  sharedptr<const LayoutItem> layout_item = m_ColumnTypes[model_column_index].m_item;
+  sharedptr<const LayoutItem_Field> layout_item_field = sharedptr<const LayoutItem_Field>::cast_dynamic(layout_item);
+
   //Tell the TreeView how to render the Gnome::Gda::Values:
-  pViewColumn->set_cell_data_func(cellrenderer, 
-    sigc::bind( sigc::mem_fun(*this, &DbAddDel::treeviewcolumn_on_cell_data), model_column_index) );
+  if(layout_item_field)
+  {
+    pViewColumn->set_cell_data_func(cellrenderer, 
+      sigc::bind( sigc::mem_fun(*this, &DbAddDel::treeviewcolumn_on_cell_data), model_column_index, data_model_column_index) );
+  }
 
   //Allow the column to be reordered by dragging and dropping the column header:
   pViewColumn->set_reorderable();
 
   //Allow the column to be resized:
   pViewColumn->set_resizable();
-
-  sharedptr<const LayoutItem> layout_item = m_ColumnTypes[model_column_index].m_item;
   
   guint column_width = 0;
   if(!layout_item->get_display_width(column_width)) //Not saved in document, but remembered when the column is resized.
@@ -1598,9 +1718,7 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
   pViewColumn->signal_clicked().connect(
     sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_column_clicked), model_column_index) );
 
-
-  pViewColumn->property_width().signal_changed().connect( 
-    sigc::bind( sigc::mem_fun(*this, &DbAddDel::on_treeview_column_resized), model_column_index, pViewColumn) );
+  pViewColumn->connect_property_changed("width", sigc::bind(sigc::mem_fun(*this, &DbAddDel::on_treeview_column_resized), model_column_index, pViewColumn) );
 
   return cols_count;
 }
@@ -1715,12 +1833,18 @@ bool DbAddDel::get_is_last_row(const Gtk::TreeModel::iterator& iter) const
 
 Gtk::TreeModel::iterator DbAddDel::get_last_row() const
 {
-  return m_refListStore->get_last_row();
+  if(m_refListStore)
+    return m_refListStore->get_last_row();
+  else
+    return Gtk::TreeModel::iterator();
 }
 
 Gtk::TreeModel::iterator DbAddDel::get_last_row()
 {
-  return m_refListStore->get_last_row();
+  if(m_refListStore)
+    return m_refListStore->get_last_row();
+  else
+    return Gtk::TreeModel::iterator();
 }
 
 Gnome::Gda::Value DbAddDel::get_value_key(const Gtk::TreeModel::iterator& iter)
@@ -1731,7 +1855,7 @@ Gnome::Gda::Value DbAddDel::get_value_key(const Gtk::TreeModel::iterator& iter)
 
 void DbAddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Gnome::Gda::Value& value)
 {
-  if(iter)
+  if(iter && m_refListStore)
   {
     Gtk::TreeModel::Row row = *iter;
 
@@ -1758,6 +1882,9 @@ bool DbAddDel::get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) cons
   if(!iter)
     return false;
 
+  if(!m_refListStore)
+    return false;
+
   if(iter == m_refListStore->children().end())
   {
     return false;
@@ -1820,27 +1947,27 @@ void DbAddDel::set_key_field(const sharedptr<Field>& field)
   m_key_field = field;
 }
 
-void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index)
+void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index, int data_model_column_index)
 {
   if(iter)
   {
-    const guint col_real = model_column_index + get_count_hidden_system_columns();
-    Gtk::TreeModel::Row treerow = *iter;
-    Gnome::Gda::Value value;
-    treerow->get_value(col_real, value);
-
     const DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
 
     sharedptr<LayoutItem_Field> field = sharedptr<LayoutItem_Field>::cast_dynamic(column_info.m_item);
     if(field)
     {
+      const guint col_real = data_model_column_index + get_count_hidden_system_columns();
+      Gtk::TreeModel::Row treerow = *iter;
+      Gnome::Gda::Value value;
+      treerow->get_value(col_real, value);
+
       switch(field->get_glom_type())
       {
         case(Field::TYPE_BOOLEAN):
         {
           Gtk::CellRendererToggle* pDerived = dynamic_cast<Gtk::CellRendererToggle*>(renderer);
           if(pDerived)
-            pDerived->set_active( (value.get_value_type() == Gnome::Gda::VALUE_TYPE_BOOLEAN) && value.get_bool() ); 
+            pDerived->set_active( (value.get_value_type() == G_TYPE_BOOLEAN) && value.get_boolean() ); 
 
           break;
         }
@@ -1855,7 +1982,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
             if(pixbuf)
               pixbuf = ImageGlom::scale_keeping_ratio(pixbuf,  get_fixed_cell_height(), pixbuf->get_width());
 
-            pDerived->property_pixbuf() = pixbuf;
+            g_object_set(pDerived->gobj(), "pixbuf", pixbuf ? pixbuf->gobj() : NULL, (gpointer)NULL);
           }
           else
             g_warning("Field::sql(): glom_type is TYPE_IMAGE but gda type is not VALUE_TYPE_BINARY");
@@ -1870,7 +1997,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
           {
             const Glib::ustring text = Conversions::get_text_for_gda_value(field->get_glom_type(), value, field->get_formatting_used().m_numeric_format);
             //g_assert(text != "NULL");
-            pDerived->property_text() = text;
+            g_object_set(pDerived->gobj(), "text", text.c_str(), (gpointer)NULL);
           }
 
           break;
@@ -1899,7 +2026,7 @@ void DbAddDel::set_allow_view_details(bool val)
 
   //Hide it if it was visible:
   if(m_treeviewcolumn_button)
-    m_treeviewcolumn_button->property_visible() = get_allow_view_details();
+    g_object_set(m_treeviewcolumn_button->gobj(), "visible", get_allow_view_details() ? TRUE : FALSE, (gpointer)NULL);
 }
 
 bool DbAddDel::get_allow_view_details() const
@@ -1909,6 +2036,9 @@ bool DbAddDel::get_allow_view_details() const
 
 void DbAddDel::on_cell_button_clicked(const Gtk::TreeModel::Path& path)
 {
+  if(!m_refListStore)
+    return;
+
   Gtk::TreeModel::iterator iter = m_refListStore->get_iter(path);
   if(iter)
   {
@@ -1918,6 +2048,14 @@ void DbAddDel::on_cell_button_clicked(const Gtk::TreeModel::Path& path)
   on_MenuPopup_activate_Edit();
 }
 
+void DbAddDel::on_self_style_changed(const Glib::RefPtr<Gtk::Style>& style)
+{
+  // Reset fixed cell height because the font might have changed due to the new style
+  m_fixed_cell_height = 0;
+  // Reconstruct columns because sizes might have changed
+  construct_specified_columns();
+}
+
 void DbAddDel::set_open_button_title(const Glib::ustring& title)
 {
   m_open_button_title = title;
