@@ -30,6 +30,8 @@
 #include <vector>
 #include <map>
 
+#include "config.h" // For GLOM_ENABLE_CLIENT_ONLY
+
 namespace Glom
 {
 
@@ -42,8 +44,9 @@ public:
   DbAddDelColumnInfo(const DbAddDelColumnInfo& src);
   DbAddDelColumnInfo& operator=(const DbAddDelColumnInfo& src);
 
-  sharedptr<const LayoutItem_Field> m_field;
+  sharedptr<LayoutItem> m_item;
 
+  //For fields with choices:
   typedef std::vector<Glib::ustring> type_vecStrings;
   type_vecStrings m_choices;
 
@@ -80,8 +83,6 @@ public:
 
   void remove_item(const Gtk::TreeModel::iterator& iter);
 
-  void remove_all();
-
   Gnome::Gda::Value get_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item);
 
   /** Get the row's hidden key
@@ -106,7 +107,7 @@ public:
    * @param start_editing Whether editing should start in the cell.
    * @result Whether the row was successfully selected.
    */
-  bool select_item(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item, bool start_editing = false);  //bool indicates success.
+  bool select_item(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem>& layout_item, bool start_editing = false);  //bool indicates success.
   bool select_item(const Gtk::TreeModel::iterator& iter, bool start_editing = false);
 
   guint get_count() const;
@@ -146,7 +147,7 @@ public:
 
   /** @result The index of the new column.
    */
-  guint add_column(const sharedptr<const LayoutItem_Field>& field);
+  guint add_column(const sharedptr<LayoutItem>& layout_item);
 
   /// Specify which records to show:
   void set_found_set(const FoundSet& found_set);
@@ -213,8 +214,10 @@ public:
   typedef sigc::signal<void, const Gtk::TreeModel::iterator&, const Gtk::TreeModel::iterator&> type_signal_user_requested_delete;
   type_signal_user_requested_delete signal_user_requested_delete();
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   typedef sigc::signal<void> type_signal_user_requested_layout;
   type_signal_user_requested_layout signal_user_requested_layout();
+#endif // !GLOM_ENABLE_CLIENT_ONLY
 
   //row number.
   typedef sigc::signal<void, const Gtk::TreeModel::iterator&> type_signal_user_requested_edit;
@@ -230,6 +233,9 @@ public:
   typedef sigc::signal<void> type_signal_user_reordered_columns;
   type_signal_user_reordered_columns signal_user_reordered_columns();
 
+  typedef sigc::signal<void, const sharedptr<const LayoutItem_Button>&, const Gtk::TreeModel::iterator&> type_signal_script_button_clicked;
+  type_signal_script_button_clicked signal_script_button_clicked();
+
  
   virtual Gtk::TreeModel::iterator get_last_row();
   virtual Gtk::TreeModel::iterator get_last_row() const;
@@ -237,13 +243,17 @@ public:
   virtual void set_open_button_title(const Glib::ustring& title);
 
 protected:
+  Gtk::CellRenderer* construct_specified_columns_cellrenderer(const sharedptr<LayoutItem>& layout_item, int model_column_index, int data_model_column_index);
 
   bool get_model_column_index(guint view_column_index, guint& model_column_index);
 
 
   typedef std::list<guint> type_list_indexes;
   ///Return the column indexes of any columns that display this field.
-  virtual type_list_indexes get_column_index(const sharedptr<const LayoutItem_Field>& layout_item) const;
+  virtual type_list_indexes get_column_index(const sharedptr<const LayoutItem>& layout_item) const;
+
+  ///Return the query column index of any columns that display this field:
+  type_list_indexes get_data_model_column_index(const sharedptr<const LayoutItem_Field>& layout_item_field) const;
 
   virtual void setup_menu();
   virtual Gnome::Gda::Value treeview_get_key(const Gtk::TreeModel::iterator& row);
@@ -252,10 +262,10 @@ protected:
   //virtual Gtk::TreeModel::iterator get_next_available_row_with_add_if_necessary();
 
   //Signal handlers:
-  void treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index);
+  void treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter, int model_column_index, int data_model_column_index);
 
-  virtual void on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index);
-  virtual void on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index);
+  virtual void on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index, int data_model_column_index);
+  virtual void on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index, int data_model_column_index);
 
   virtual bool on_treeview_column_drop(Gtk::TreeView* treeview, Gtk::TreeViewColumn* column, Gtk::TreeViewColumn* prev_column, Gtk::TreeViewColumn* next_column);
   virtual void on_treeview_columns_changed();
@@ -265,13 +275,24 @@ protected:
   virtual void on_MenuPopup_activate_Edit();
   virtual void on_MenuPopup_activate_Add();
   virtual void on_MenuPopup_activate_Delete();
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   virtual void on_MenuPopup_activate_layout();
+#endif
 
   virtual void on_treeview_button_press_event(GdkEventButton* event);
 
   virtual bool on_treeview_columnheader_button_press_event(GdkEventButton* event);
   virtual void on_treeview_column_clicked(int model_column_index);
+  void on_treeview_column_resized(int model_column_index, DbTreeViewColumnGlom* view_column);
   virtual void on_cell_button_clicked(const Gtk::TreeModel::Path& path);
+  void on_cell_layout_button_clicked(const Gtk::TreeModel::Path& path, int model_column_index);
+
+  // Don't call it on_style_changed, otherwise we would override a virtual
+  // function from Gtk::Widget. We could indeed do that, but we do it with
+  // a normal signal handler, because we have to do it this way anyway in
+  // case default signal handlers have been disabled in glibmm.
+  void on_self_style_changed(const Glib::RefPtr<Gtk::Style>& style);
 
   bool get_prevent_user_signals() const;
 
@@ -283,7 +304,15 @@ protected:
   guint get_count_hidden_system_columns();
 
   //The column_id is extra information that we can use later to discover what the column shows, even when columns have been reordered.
-  guint treeview_append_column(const Glib::ustring& title, Gtk::CellRenderer& cellrenderer, int model_column_index);
+  guint treeview_append_column(const Glib::ustring& title, Gtk::CellRenderer& cellrenderer, int model_column_index, int data_model_column_index);
+
+  /** Show a model that gives a visual hint to the developer,
+   * when he has not yet specified fields to show.
+   * TODO: Make this more obvious in a less strange way.
+   */
+  void show_hint_model();
+
+  int get_fixed_cell_height();
 
   App_Glom* get_application();
 
@@ -313,7 +342,11 @@ protected:
   Gtk::Menu* m_pMenuPopup;
   Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;
   Glib::RefPtr<Gtk::UIManager> m_refUIManager;
-  Glib::RefPtr<Gtk::Action> m_refContextEdit, m_refContextAdd, m_refContextDelete, m_refContextLayout;
+  Glib::RefPtr<Gtk::Action> m_refContextEdit, m_refContextAdd, m_refContextDelete;
+
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+  Glib::RefPtr<Gtk::Action> m_refContextLayout;
+#endif
 
   bool m_bAllowUserActions;
 
@@ -339,9 +372,12 @@ protected:
   type_signal_user_requested_delete m_signal_user_requested_delete;
   type_signal_user_requested_edit m_signal_user_requested_edit;
   type_signal_user_requested_add m_signal_user_requested_add;
+#ifndef GLOM_ENABLE_CLIENT_ONLY
   type_signal_user_requested_layout m_signal_user_requested_layout;
+#endif // !GLOM_ENABLE_CLIENT_ONLY
   type_signal_user_activated m_signal_user_activated;
   type_signal_user_reordered_columns m_signal_user_reordered_columns;
+  type_signal_script_button_clicked m_signal_script_button_clicked;
 
   bool get_ignore_treeview_signals() const;
   void set_ignore_treeview_signals(bool ignore = true);
@@ -366,8 +402,24 @@ protected:
     type_vecColumns m_vecColumns;
   };
 */
-};
+
+  //When no columns have been chosen in the layout editor,
+  //show this model to give the user a hint about what to do:
+  class ModelColumnsEmptyHint : public Gtk::TreeModel::ColumnRecord
+  {
+  public:
+
+    ModelColumnsEmptyHint()
+    { add(m_col_hint); }
+
+    Gtk::TreeModelColumn<Glib::ustring> m_col_hint;
+  };
  
+  ModelColumnsEmptyHint m_columns_hint;
+  Glib::RefPtr<Gtk::ListStore> m_model_hint;
+
+  int m_fixed_cell_height;
+};
 
 } //namespace Glom
 
